﻿// cc-by 3.0 Austria, (c) 2011 Markus Loeberbauer
// http://ssw.jku.at/Teaching/Lectures/SpezialLVA/JavaFX/
package at.ssw.jku;

import javafx.animation.Animation.Status;
import javafx.animation.KeyFrame;
import javafx.animation.KeyValue;
import javafx.animation.Timeline;
import javafx.animation.TimelineBuilder;
import javafx.application.Application;
import javafx.beans.property.DoubleProperty;
import javafx.beans.property.SimpleDoubleProperty;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.Group;
import javafx.scene.GroupBuilder;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.input.KeyEvent;
import javafx.scene.transform.Rotate;
import javafx.scene.transform.Scale;
import javafx.stage.Stage;
import javafx.util.Duration;
import at.ssw.jku.controller.GameLogic;
import at.ssw.jku.model.Direction;
import at.ssw.jku.model.GameState;
import at.ssw.jku.view.FieldNode;

public final class Worms extends Application {
  public static void main(String[] args) {
    Application.launch(args);
  }
  private Stage stage;
  private GameLogic game = new GameLogic();
  private FieldNode fieldNode = new FieldNode(game.getField());
  private EventHandler<KeyEvent> handleKeyEvent = new EventHandler<KeyEvent>() {
    @Override
    public void handle(KeyEvent evt) {
      switch (evt.getCode()) {
        case UP:
          game.setWormDirection(Direction.UP);
          break;
        case DOWN:
          game.setWormDirection(Direction.DOWN);
          break;
        case LEFT:
          game.setWormDirection(Direction.LEFT);
          break;
        case RIGHT:
          game.setWormDirection(Direction.RIGHT);
          break;
        case ENTER:
          if (evt.isAltDown()) {
            stage.setFullScreen(!stage.isFullScreen());
          }
          break;
        case P:
        case SPACE:
          game.togglePause();
          break;
        case R:
          toggleRotate();
          break;
      }
    }
  };
  private final DoubleProperty rotateAngle = new SimpleDoubleProperty();
  private final Timeline rotateAnim = TimelineBuilder.create().cycleCount(Timeline.INDEFINITE).keyFrames(new KeyFrame(Duration.ZERO, new KeyValue(rotateAngle, 0)), new KeyFrame(Duration.seconds(20), new KeyValue(rotateAngle, 360))).build();
  
  private void toggleRotate() {
    if (rotateAnim.getStatus() != Status.RUNNING || rotateAnim.getStatus() == Status.PAUSED) {
      rotateAnim.play();
    } else {
      rotateAnim.pause();
      rotateAngle.set(0);
      rotateAnim.jumpTo(Duration.ZERO);
    }
  }
  
  @Override
  public void start(Stage primaryStage) {
    stage = primaryStage;
    stage.setTitle("Worm");
    stage.setWidth(600);
    stage.setHeight(600);
    
    final Button playButton = new Button("Play");
    playButton.setOnAction(new EventHandler<ActionEvent>() {
      @Override
      public void handle(ActionEvent t) {
        game.togglePause();
      }
    });
    
    final Group sceneGroup = GroupBuilder.create().children(
            fieldNode, playButton).onKeyPressed(handleKeyEvent).build();
    
    final Scene scene = new Scene(sceneGroup);
    
    playButton.layoutXProperty().bind(scene.widthProperty().subtract(playButton.widthProperty()).divide(2));
    playButton.layoutYProperty().bind(scene.heightProperty().subtract(playButton.heightProperty()).divide(2));
    
    Scale scale = new Scale();
    scale.xProperty().bind(scene.widthProperty().divide(fieldNode.getWidth()));
    scale.yProperty().bind(scene.heightProperty().divide(fieldNode.getHeight()));
    
    fieldNode.getTransforms().add(scale);
    
    final Rotate rotate = new Rotate(0, fieldNode.getWidth() / 2, fieldNode.getHeight() / 2);
    rotate.angleProperty().bind(rotateAngle);
    
    fieldNode.getTransforms().add(rotate);
    
    game.stateProperty().addListener(new ChangeListener<GameState>() {
      @Override
      public void changed(ObservableValue<? extends GameState> observable, GameState oldValue, GameState newValue) {
        switch (newValue) {
          case RUNNING:
            sceneGroup.requestFocus();
            playButton.setVisible(false);
            break;
          case PAUSED:
            playButton.setVisible(true);
            break;
          case GAMEOVER:
            playButton.setVisible(true);
            break;
          default:
            throw new IllegalArgumentException("Unknown game state: " + newValue);
        }
      }
    });
    
    stage.focusedProperty().addListener(new ChangeListener<Boolean>() {
      @Override
      public void changed(ObservableValue<? extends Boolean> observable, Boolean oldValue, Boolean newValue) {
        if (newValue) {
          sceneGroup.requestFocus();
        }
      }
    });
    
    sceneGroup.focusedProperty().addListener(new ChangeListener<Boolean>() {
      @Override
      public void changed(ObservableValue<? extends Boolean> observable, Boolean oldValue, Boolean newValue) {
        if (!newValue && game.getState() != GameState.GAMEOVER) {
          game.pause();
        }
      }
    });
    
    stage.setScene(scene);
    stage.show();
  }
}
