﻿// cc-by 3.0 Austria, (c) 2011 Markus Loeberbauer
// http://ssw.jku.at/Teaching/Lectures/SpezialLVA/JavaFX/
package at.ssw.jku.controller;

import java.util.Random;

import javafx.animation.KeyFrame;
import javafx.animation.Timeline;
import javafx.animation.TimelineBuilder;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.ReadOnlyObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.util.Duration;
import at.ssw.jku.model.Direction;
import at.ssw.jku.model.Field;
import at.ssw.jku.model.GameState;
import at.ssw.jku.model.Goody;
import at.ssw.jku.model.Segment;
import at.ssw.jku.model.Worm;

public final class GameLogic {
  private final Random random = new Random();
  private final Goody goody = new Goody();
  private final Worm worm = new Worm(Field.COLUMNS / 2, Field.ROWS / 2);
  private final Field field = new Field(worm, goody);
  private final Timeline anim = TimelineBuilder.create().cycleCount(Timeline.INDEFINITE).
      keyFrames(new KeyFrame(Duration.millis(40), new EventHandler<ActionEvent>() {
    @Override
    public void handle(ActionEvent t) {
      worm.move();
      checkGame();
      plantGoody();
    }
  })).cycleCount(Timeline.INDEFINITE).build();
  private ObjectProperty<GameState> gameStateProperty = new SimpleObjectProperty<GameState>();

  public GameLogic() {
    pause();
  }

  private void checkGame() {
    if (isWormOnWall() || isWormOnWorm()) {
      handleGameOver();
    } else if (isWormOnGoody()) {
      worm.grow(goody.getValue());
      goody.consume();
    }
  }

  private void handleGameOver() {
    anim.stop();
    setState(GameState.GAMEOVER);
  }

  private void plantGoody() {
    if (!goody.isAvailable()) {
      goody.plant(random.nextInt(Field.COLUMNS), random.nextInt(Field.ROWS), goody.
          getValue() + 1);
    }
  }

  private boolean isWormOnWall() {
    Segment head = worm.getSegments().get(0);
    return head.getX() < 0 || head.getY() < 0 || head.getX() >= Field.COLUMNS || head.
        getY() >= Field.ROWS;
  }

  private boolean isWormOnWorm() {
    Segment head = worm.getSegments().get(0);
    for (Segment seg : worm.getSegments().subList(1, worm.getSegments().size())) {
      if (head.getX() == seg.getX() && head.getY() == seg.getY()) {
        return true;
      }
    }
    return false;
  }

  private boolean isWormOnGoody() {
    Segment head = worm.getSegments().get(0);
    return goody.isAvailable() && goody.getX() == head.getX()
        && goody.getY() == head.getY();
  }

  public void togglePause() {
    final GameState state = getState();
    switch (state) {
      case GAMEOVER:
        start();
        break;
      case RUNNING:
        pause();
        break;
      case PAUSED:
        resume();
        break;
      default:
        throw new IllegalArgumentException("Unknown game state: " + state);
    }
  }

  public void start() {
    worm.reset();
    goody.reset();
    resume();
  }

  void resume() {
    setState(GameState.RUNNING);
    anim.play();
  }

  public void pause() {
    anim.pause();
    setState(GameState.PAUSED);
  }

  public void setWormDirection(Direction dir) {
    if (worm.getDirection() != dir.opposite()) {
      worm.setDirection(dir);
    }
  }

  public Field getField() {
    return field;
  }

  public GameState getState() {
    return gameStateProperty.get();
  }

  private void setState(GameState state) {
    gameStateProperty.set(state);
  }

  public ReadOnlyObjectProperty<GameState> stateProperty() {
    return gameStateProperty;
  }
}