﻿// cc-by 3.0 Austria, (c) 2011 Markus Loeberbauer
// http://ssw.jku.at/Teaching/Lectures/SpezialLVA/JavaFX/
package at.ssw.jku.model;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;

public final class Worm {
  private final int startX;
  private final int startY;
  private Direction direction = Direction.UP;
  private ObservableList<Segment> segments = FXCollections.observableArrayList();
  private int segmentsToGrow;

  public Worm(int startX, int startY) {
    this.startX = startX;
    this.startY = startY;
    reset();
  }

  public void move() {
    advanceHead();
    advanceTail();
  }

  private void advanceHead() {
    if (segments.isEmpty()) {
      return;
    }
    Segment curHead = segments.get(0);
    Segment newHead;

    switch (direction) {
      case UP:
        newHead = new Segment(curHead.getX(), curHead.getY() - 1);
        break;
      case DOWN:
        newHead = new Segment(curHead.getX(), curHead.getY() + 1);
        break;
      case LEFT:
        newHead = new Segment(curHead.getX() - 1, curHead.getY());
        break;
      case RIGHT:
        newHead = new Segment(curHead.getX() + 1, curHead.getY());
        break;
      default:
        throw new IllegalArgumentException("Unknown value: {direction}");

    }

    segments.add(0, newHead);
  }

  private void advanceTail() {
    if (segments.isEmpty()) {
      return;
    }
    if (segmentsToGrow == 0) {
      segments.remove(segments.size() - 1);
    } else {
      segmentsToGrow--;
    }
  }

  public void grow(int bySegments) {
    segmentsToGrow += bySegments;
  }

  public void reset() {
    segments.clear();
    segments.add(new Segment(startX, startY));
  }
  
  public ObservableList<Segment> getSegments() {
    return segments;
  }
  
  public Direction getDirection() {
    return direction;
  }
  
  public void setDirection(Direction newDirection) {
    direction = newDirection;
  }
}
